using System;
using System.Management;
using System.Threading;
using System.Configuration;
using System.Data.SqlClient;
using System.Data;
using System.Data.Common;
using System.IO;
using System.Diagnostics;
using System.Messaging;
using System.Runtime.Serialization.Formatters.Soap;
using System.Text;

#if !NUNIT
using Microsoft.VisualStudio.TestTools.UnitTesting;
#else
using NUnit.Framework;
using TestClass = NUnit.Framework.TestFixtureAttribute;
using TestInitialize = NUnit.Framework.SetUpAttribute;
using TestCleanup = NUnit.Framework.TearDownAttribute;
using TestMethod = NUnit.Framework.TestAttribute;
#endif
using Microsoft.Practices.EnterpriseLibrary.ExceptionHandling;
using Microsoft.Practices.EnterpriseLibrary.Logging;
using Microsoft.Practices.EnterpriseLibrary.Data;
using Microsoft.Practices.EnterpriseLibrary.Logging.Database;



namespace ExceptionHandlingLoggingTests
{
    /// <summary>
    /// Summary description for LoggingDBTestFixture
    /// </summary>
    [TestClass]
    public class LoggingDBTestFixture
    {
        string connString = "server=.\\sqlexpress;database=Logging;Integrated Security=true";
        public LoggingDBTestFixture()
        {
        }

        [TestInitialize()]
        public void Initialize()
        {
            Microsoft.Practices.EnterpriseLibrary.Data.Database db = DatabaseFactory.CreateDatabase();
            DbCommand command = db.GetStoredProcCommand("ClearLogs");
            db.ExecuteNonQuery(command);
         }

        #region Additional test attributes
        //
        // You can use the following additional attributes as you write your tests:
        //
        // Use ClassInitialize to run code before running the first test in the class
        // [ClassInitialize()]
        // public static void MyClassInitialize(TestContext testContext) { }
        //
        // Use ClassCleanup to run code after all tests in a class have run
        // [ClassCleanup()]
        // public static void MyClassCleanup() { }
        //
        // Use TestInitialize to run code before running each test 
        // [TestInitialize()]
        // public void MyTestInitialize() { }
        //
        // Use TestCleanup to run code after each test has run
        // [TestCleanup()]
        // public void MyTestCleanup() { }
        //
        #endregion

        #region "Database"

        [TestMethod]
        public void LoggingNoneActionDBSinkTest()
        {
            Exception originalException = new System.Security.SecurityException("Security Exception logged in database");
            bool rethrow = false;
            rethrow = ExceptionPolicy.HandleException(originalException, "Log To Database PostHandlingAction None");

            Assert.IsFalse(rethrow);
            string title = null;
            using (SqlConnection dbConnection = new SqlConnection(connString))
            {
                dbConnection.Open();
                SqlCommand cmdRetrieveData = new SqlCommand("select title from log where eventid=113", dbConnection);

                object retValue = cmdRetrieveData.ExecuteScalar();
                if (retValue == null)
                {
                    Assert.Fail("No Log Entry in Database");
                }

                cmdRetrieveData.Dispose();

                title = (string)retValue;
            }

            Assert.AreEqual("Log To Database PostHandlingAction None", title);


        }

        [TestMethod]
        public void LoggingThrowNewDBSinkTest()
        {
            bool exceptionRaised = false;
            bool rethrow = false;
            Exception originalException = new System.Security.SecurityException("Security Exception logged in database");
            try
            {
                ExceptionPolicy.HandleException(originalException, "Log To Database PostHandlingAction Throw New Exception");
                if (rethrow) throw originalException;

            }
            catch (Exception e)
            {
                exceptionRaised = true;
                Assert.AreEqual("Security Exception logged in database", e.Message);
                Assert.AreEqual("SecurityException", e.GetType().Name);

            }
            Assert.IsFalse(rethrow);

            string title;
            using (SqlConnection dbConnection = new SqlConnection(connString))
            {
                dbConnection.Open();
                SqlCommand cmdRetrieveData = new SqlCommand("select title from log where eventid=115", dbConnection);

                object retValue = cmdRetrieveData.ExecuteScalar();
                if (retValue == null)
                {
                    Assert.Fail("No Log Entry in Database");
                }

                cmdRetrieveData.Dispose();

                title = (string)retValue;
            }


            Assert.AreEqual("Log To Database PostHandlingAction Throw New Exception", title);
            Assert.AreEqual(true, exceptionRaised);

        }

        [TestMethod]
        public void LoggingRethrowDBSinkTest()
        {
            Exception originalException = new System.IO.FileNotFoundException("File Not Found Exception logged in database");
            bool rethrow = ExceptionPolicy.HandleException(originalException, "Log To Database PostHandlingAction Rethrow Exception");

            Assert.IsTrue(rethrow);

            string title = null;
            using (SqlConnection dbConnection = new SqlConnection(connString))
            {
                dbConnection.Open();
                SqlCommand cmdRetrieveData = new SqlCommand("select title from log where eventid=114", dbConnection);

                object retValue = cmdRetrieveData.ExecuteScalar();
                if (retValue == null)
                {
                    Assert.Fail("No Log Entry in Database");
                }

                cmdRetrieveData.Dispose();

                title = (string)retValue;
            }
            Assert.AreEqual("Log To Database PostHandlingAction Rethrow Exception", title);
            
        }

        [TestMethod]
        public void LoggingDiffSettingsDBSinkTest()
        {
            Exception originalException = new System.IO.FileNotFoundException("File Not Found Exception logged in database");
            bool rethrow = ExceptionPolicy.HandleException(originalException, "Logging Database Diff Settings");

            SqlConnection dbConnection = new SqlConnection(connString);
            dbConnection.Open();
            SqlCommand cmdRetrieveData = new SqlCommand("select title from log where eventid=110 and priority=1 and severity='Error'", dbConnection);
            string title;


            title = cmdRetrieveData.ExecuteScalar().ToString();

            dbConnection.Close();
            Assert.AreEqual("Enterprise Library Logging - Error", title);


        }


        #endregion

        #region "Multiple Handlers"

        [TestMethod]
        public void LoggingWrapDBSinkSameAssemblyTest()
        {
            bool exceptionRaised = false;
            Exception originalException = new System.Security.SecurityException("Security exception 1 logged into database");
            bool rethrow = false;
            try
            {
                rethrow = ExceptionPolicy.HandleException(originalException, "Log To Database Wrap With Same Assembly");
                if (rethrow) throw originalException;

            }
            catch (Exception e)
            {
                exceptionRaised = true;
                Assert.AreEqual("Security Exception 1 is logged and is wrapped by Verification Exception", e.Message);
                Assert.AreEqual("Security exception 1 logged into database", e.InnerException.Message);
                Assert.AreEqual("VerificationException", e.GetType().Name);
                Assert.AreEqual("SecurityException", e.InnerException.GetType().Name);

            }

            Assert.IsFalse(rethrow);

            Assert.AreEqual(true, exceptionRaised);
            SqlConnection dbConnection = new SqlConnection(connString);
            dbConnection.Open();
            SqlCommand cmdRetrieveData = new SqlCommand("select title from log where eventid=115 and priority=0 and severity='Error'", dbConnection);
            string title;


            title = cmdRetrieveData.ExecuteScalar().ToString();

            dbConnection.Close();
            Assert.AreEqual("Exception Logging and Wrapping", title);
        }

        [TestMethod]
        public void LoggingWrapDBSinkDiffAssemblyTest()
        {
            bool exceptionRaised = false;
            Exception originalException = new System.IO.FileNotFoundException("File Not Found exception 1 logged into database");
            bool rethrow = false;
            try
            {
                rethrow = ExceptionPolicy.HandleException(originalException, "Log To Database Wrap With Diff Assembly");
                if (rethrow) throw originalException;
            }
            catch (Exception e)
            {
                exceptionRaised = true;
                Assert.AreEqual("IO Exception 1 is logged and is wrapped by Security Exception", e.Message);
                Assert.AreEqual("File Not Found exception 1 logged into database", e.InnerException.Message);

            }

            Assert.IsFalse(rethrow);


            Assert.AreEqual(true, exceptionRaised);
            SqlConnection dbConnection = new SqlConnection(connString);
            dbConnection.Open();
            SqlCommand cmdRetrieveData = new SqlCommand("select title from log where eventid=114 and priority=0 and severity='Error'", dbConnection);
            string title;


            title = cmdRetrieveData.ExecuteScalar().ToString();

            dbConnection.Close();
            Assert.AreEqual("Exception Logging and Wrapping", title);


        }

        [TestMethod]
        public void LoggingReplaceDBSinkSameAssemblyTest()
        {
            bool exceptionRaised = false;
            Exception originalException = new System.Security.SecurityException("Security exception 1 logged into database");
            bool rethrow = false;
            try
            {
                rethrow = ExceptionPolicy.HandleException(originalException, "Log To Database Replace With Same Assembly");
                if (rethrow) throw originalException;
            }
            catch (Exception e)
            {
                exceptionRaised = true;
                Assert.AreEqual("Security Exception 1 is logged and is replaced by Verification Exception", e.Message);
            }

            Assert.IsFalse(rethrow);
            Assert.AreEqual(true, exceptionRaised);
            SqlConnection dbConnection = new SqlConnection(connString);
            dbConnection.Open();
            SqlCommand cmdRetrieveData = new SqlCommand("select title from log where eventid=121 and priority=0 and severity='Information'", dbConnection);
            string title;


            title = cmdRetrieveData.ExecuteScalar().ToString();

            dbConnection.Close();
            Assert.AreEqual("Exception Logging and Replacing", title);


        }

        [TestMethod]
        public void LoggingReplaceDBSinkDiffAssemblyTest()
        {
            bool exceptionRaised = false;
            Exception originalException = new System.IO.FileNotFoundException("File Not Found exception 1 logged into database");
            bool rethrow = false;
            try
            {
                rethrow = ExceptionPolicy.HandleException(originalException, "Log To Database Replace With Diff Assembly");
                if (rethrow) throw originalException;
            }
            catch (Exception e)
            {
                exceptionRaised = true;
                Assert.AreEqual("Security Exception 1 is logged and is replaced by IO Exception", e.Message);
            }

            Assert.IsFalse(rethrow);
            Assert.AreEqual(true, exceptionRaised);
            SqlConnection dbConnection = new SqlConnection(connString);
            dbConnection.Open();
            SqlCommand cmdRetrieveData = new SqlCommand("select title from log where eventid=120 and priority=0 and severity='Error'", dbConnection);
            string title;


            title = cmdRetrieveData.ExecuteScalar().ToString();

            dbConnection.Close();
            Assert.AreEqual("Exception Logging and Replacing", title);


        }

        [TestMethod]
        public void ReplaceLogToDBSinkNoneActionTest()
        {
            Exception originalException = new System.Security.SecurityException("Original Exception: Security Exception Replaced by IO Exception which is logged into DB sink");

            bool rethrow = ExceptionPolicy.HandleException(originalException, "Replace And Log To DB With PostHandlingAction None");

            Assert.IsFalse(rethrow);
            string title = null;
            using (SqlConnection dbConnection = new SqlConnection(connString))
            {
                dbConnection.Open();
                SqlCommand cmdRetrieveData = new SqlCommand("select title from log where eventid=223", dbConnection);

                object retValue = cmdRetrieveData.ExecuteScalar();
                if (retValue == null)
                {
                    Assert.Fail("No Log Entry in Database");
                }

                cmdRetrieveData.Dispose();

                title = (string)retValue;
            }

            Assert.AreEqual("Replace And Log To DB With PostHandlingAction None", title);

        }

        [TestMethod]
        public void ReplaceLogToDBThrowNewTest()
        {
            bool exceptionRaised = false;
            Exception originalException = new System.Security.SecurityException("Original Exception: Security Exception wrapped by IO Exception which is logged into DB sink");

            bool rethrow = false;
            try
            {
                rethrow = ExceptionPolicy.HandleException(originalException, "Replace And Log To DB With PostHandlingAction Throw New Exception");
                if (rethrow) throw originalException;
            }
            catch (Exception e)
            {
                exceptionRaised = true;
                Assert.AreEqual("Security exception replaced by IO exception", e.Message);
                Assert.AreEqual("IOException", e.GetType().Name);

            }
            Assert.IsFalse(rethrow);
            string title = null;
            using (SqlConnection dbConnection = new SqlConnection(connString))
            {
                dbConnection.Open();
                SqlCommand cmdRetrieveData = new SqlCommand("select title from log where eventid=224", dbConnection);

                object retValue = cmdRetrieveData.ExecuteScalar();
                if (retValue == null)
                {
                    Assert.Fail("No Log Entry in Database");
                }

                cmdRetrieveData.Dispose();

                title = (string)retValue;
            }

            Assert.AreEqual("Replace And Log To DB With PostHandling Throw New Exception", title);
            Assert.AreEqual(true, exceptionRaised);

        }

        [TestMethod]
        public void WrapLogToDBSinkNoneActionTest()
        {
            Exception originalException = new System.Security.SecurityException("Security exception 1 logged into database");
            bool rethrow = false;

            rethrow = ExceptionPolicy.HandleException(originalException, "Wrap And Log To DB With PostHandlingAction None");
            if (rethrow) throw originalException;

            Assert.IsFalse(rethrow);
            
            SqlConnection dbConnection = new SqlConnection(connString);
            dbConnection.Open();
            SqlCommand cmdRetrieveData = new SqlCommand("select title from log where eventid=132 and priority=2 and severity='Information'", dbConnection);
            string title;


            title = cmdRetrieveData.ExecuteScalar().ToString();

            dbConnection.Close();
            Assert.AreEqual("Wrapping and Logging - Information", title);

        }

        [TestMethod]
        public void WrapLogToDBThrowNewTest()
        {
            bool exceptionRaised = false;
            Exception originalException = new System.Security.SecurityException("Original Exception: Security Exception wrapped by IO Exception which is logged into DB sink");
            bool rethrow = false;
            try
            {
                rethrow = ExceptionPolicy.HandleException(originalException, "Wrap And Log To DB With PostHandlingAction Throw New Exception");
                if (rethrow) throw originalException;

            }
            catch (Exception e)
            {
                exceptionRaised = true;
                Assert.AreEqual("Security exception 6 wrapped by IO exception", e.Message);
                Assert.AreEqual("Original Exception: Security Exception wrapped by IO Exception which is logged into DB sink", e.InnerException.Message);
                Assert.AreEqual("IOException", e.GetType().Name);
                Assert.AreEqual("SecurityException", e.InnerException.GetType().Name);

            }

            Assert.IsFalse(rethrow);

            Assert.AreEqual(true, exceptionRaised);
            SqlConnection dbConnection = new SqlConnection(connString);
            dbConnection.Open();
            SqlCommand cmdRetrieveData = new SqlCommand("select title from log where eventid=133 and priority=3 and severity='Warning'", dbConnection);
            string title;


            title = cmdRetrieveData.ExecuteScalar().ToString();

            dbConnection.Close();
            Assert.AreEqual("Wrapping and Logging - Warning", title);

        }
        #endregion

    }
}
