using System;
using System.Text;
using System.Collections.Generic;
#if !NUNIT
using Microsoft.VisualStudio.TestTools.UnitTesting;
#else
using NUnit.Framework;
using TestClass = NUnit.Framework.TestFixtureAttribute;
using TestInitialize = NUnit.Framework.SetUpAttribute;
using TestCleanup = NUnit.Framework.TearDownAttribute;
using TestMethod = NUnit.Framework.TestAttribute;
#endif
using System.Diagnostics;
using Microsoft.Practices.EnterpriseLibrary.Common.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Logging;
using Microsoft.Practices.EnterpriseLibrary.Logging.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Common;
using Microsoft.Practices.EnterpriseLibrary.Logging.TraceListeners;
using Microsoft.Practices.EnterpriseLibrary.Logging.Formatters;
using Microsoft.Practices.EnterpriseLibrary.Logging.ExtraInformation;
using Microsoft.Practices.EnterpriseLibrary.Common.Configuration.ObjectBuilder;
using CustomClasses;
namespace ConfigurationSourceTests
{
    /// <summary>
    /// Summary description for UnitTest1
    /// </summary>
    [TestClass]
    public class DictionarySourceFiltersTestFixture
    {
        private DictionaryConfigurationSource source;

        public DictionarySourceFiltersTestFixture()
        {
            source = LoggingConfigurationSource.BuildConfigurationSource();  
        }

        #region Additional test attributes
        //
        // You can use the following additional attributes as you write your tests:
        //
        // Use ClassInitialize to run code before running the first test in the class
        // [ClassInitialize()]
        // public static void MyClassInitialize(TestContext testContext) { }
        //
        // Use ClassCleanup to run code after all tests in a class have run
        // [ClassCleanup()]
        // public static void MyClassCleanup() { }
        //
        // Use TestInitialize to run code before running each test 
        // [TestInitialize()]
        // public void MyTestInitialize() { }
        //
        // Use TestCleanup to run code after each test has run
        // [TestCleanup()]
        // public void MyTestCleanup() { }
        //
        #endregion


        #region CategoryFilter Tests
        /// <summary>
        /// Test to verify the category filter mode AllowAllExceptDenied
        /// </summary>
        [TestMethod]
        public void AllowAllCategoriesTest()
        {
            LogEntry log = new LogEntry();
            log.Message = "AllowAllCategoriesTest";
            log.Categories.Add("General");
            Assert.IsTrue(IsLogEntryLoggedToEventLog(log));
        }

        /// <summary>
        ///If one of the categories to be logged is added to AllowAllExceptDenied mode, 
        ///then the message would not be logged and filtered out. 
        /// </summary>
        [TestMethod]
        public void DenyCategoryTest()
        {
            LogEntry log = new LogEntry();
            log.Message = "DenyCategoryTest";
            log.Categories.Add("General");
            log.Categories.Add("DenyCategory");
            Assert.IsFalse(IsLogEntryLoggedToEventLog(log));
        }
        #endregion

        #region PriorityFilter Tests

        /// <summary>
        /// To test the minimum priority of the filter
        /// </summary>
        [TestMethod]
        public void MinimumPriorityFilterTest()
        {
            LogEntry log = new LogEntry();
            log.Priority = 1;
            Assert.IsTrue(IsLogEntryLoggedToEventLog(log));
        }

        /// <summary>
        /// To test the maximum priority of the filter
        /// </summary>
        [TestMethod]
        public void MaximumPriorityFilterTest()
        {
            LogEntry log = new LogEntry();
            log.Priority = 4;
            Assert.IsTrue(IsLogEntryLoggedToEventLog(log));
        }

        /// <summary>
        /// To test the log message is filtered when the priority if out of range
        /// </summary>
        [TestMethod]
        public void OutofRangePriorityFilterTest()
        {
            LogEntry log = new LogEntry();
            log.Priority = 5;
            Assert.IsFalse(IsLogEntryLoggedToEventLog(log));
        }

        /// <summary>
        /// To test the log message is filtered when the piority is below range
        /// </summary>
        [TestMethod]
        public void BelowRangePriorityFilterTest()
        {
            LogEntry log = new LogEntry();
            log.Priority = 0;
            Assert.IsFalse(IsLogEntryLoggedToEventLog(log));
        }

        #endregion

        #region LoggingEnabledFilter Tests
        /// <summary>
        /// To test the log message is logged when the loggingEnabled is set to true
        /// </summary>
        [TestMethod]
        public void LoggingEnabledFilterTest()
        {
            LogEntry log = new LogEntry();
            log.Message = "LoggingEnabledFilterTest";
            Assert.IsTrue(IsLogEntryLoggedToEventLog(log));
        }

        [TestMethod]
        public void LoggingDisabledFilterTest()
        {
            LoggingSettings setting = (LoggingSettings)source.GetSection("loggingConfiguration");
            LogEnabledFilterData filter = (LogEnabledFilterData)setting.LogFilters.Get("LogEnabledFilter");
            filter.Enabled = false;
            LogEntry log = new LogEntry();
            log.Message = "LoggingDisabledFilterTest";
            Assert.IsFalse(IsLogEntryLoggedToEventLog(log));
            filter.Enabled = true;
        }

        #endregion

        #region CustomFilter Tests
        /// <summary>
        /// To test the custom filter does not log the message that is not specified in the configuration file
        /// </summary>
        [TestMethod]
        public void DoNotLogCustomSeverityFilterTest()
        {
            LogEntry log = new LogEntry();
            log.Severity = TraceEventType.Verbose;
            Assert.IsFalse(IsLogEntryLoggedToEventLog(log));
        }

        /// <summary>
        /// To test the custom filter logs only the messages with severity specified in the configuration
        /// </summary>
        [TestMethod]
        public void LogCustomSeverityFilterTest()
        {
            LogEntry log = new LogEntry();
            log.Severity = TraceEventType.Information;
            Assert.IsTrue(IsLogEntryLoggedToEventLog(log));
        }

        #endregion

        #region Helper function
        private bool IsLogEntryLoggedToEventLog(LogEntry log)
        {
            string logName = "Application";
            if (EventLog.SourceExists("Enterprise Library Logging Test"))
            {
                logName = EventLog.LogNameFromSourceName("Enterprise Library Logging Test", ".");
            }
            else
            {
                EventLog.CreateEventSource("Enterprise Library Logging Test", "Application");
            }
            using (EventLog eventLog = new EventLog(logName))
            {
                int initialCount = eventLog.Entries.Count;
                LogWriterFactory factory = new LogWriterFactory(source);
                using (LogWriter writer = factory.Create())
                {
                    writer.Write(log);
                }
                int finalCount = eventLog.Entries.Count;
                string sourceName = eventLog.Entries[finalCount - 1].Source;
                string message = eventLog.Entries[finalCount - 1].Message;
                return ((finalCount - initialCount == 1) && (sourceName.Equals("Enterprise Library Logging Test")) && message.Contains(log.Message));
            }
        }
        #endregion
    }
}
